﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.UI;
using Telerik.Web.UI;
using Telerik.Windows.Documents.Flow.FormatProviders.Docx;
using Telerik.Windows.Documents.Flow.Model;
using Telerik.Windows.Documents.Flow.Model.Styles;
using Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx;
using Telerik.Windows.Documents.Spreadsheet.Model;

public partial class _RadTreeList : System.Web.UI.Page
{
    static TreeListItemType[] supportedItemTypes = new TreeListItemType[] 
                { 
                    TreeListItemType.HeaderItem, 
                    TreeListItemType.AlternatingItem, 
                    TreeListItemType.Item 
                };

    protected void RadGrid1_ItemCommand(object sender, GridCommandEventArgs e)
    {
        if (e.CommandName == "ExportToExcel")
        {
            this.PreRender += (s, a) => { ExportToXlsx(); };
            e.Canceled = true;
        }
        else if (e.CommandName == "ExportToWord")
        {
            this.PreRenderComplete += (s, a) => { ExportToDocx(); };
            e.Canceled = true;
        }
    }

    private void ExportToDocx()
    {
        string fileName = "RadTreeList.docx";
        string filePath = Server.MapPath(fileName);

        PrepareForExport();

        RadFlowDocument flowDoc = new RadFlowDocument();
        Section section = flowDoc.Sections.AddSection();
        Table mainTable = section.Blocks.AddTable();

        TreeListItem item;
        TableRow row;

        var treeListItems = RadTreeList1.GetItems(supportedItemTypes);
        int maxLevel = RadTreeList1.GetMaximumNestedLevel() + 1;

        for (int itemNum = 0; itemNum < treeListItems.Length; itemNum++)
        {
            item = treeListItems[itemNum];
            row = mainTable.Rows.AddTableRow();

            for (int cellNum = 0; cellNum < item.Cells.Count; cellNum++)
            {
                var currentCell = item.Cells[cellNum];
                var dataItem = item as TreeListDataItem;
                string cellText = currentCell.Text;
                TableCell td = row.Cells.AddTableCell();

                var currentGridCell = item.Cells[cellNum];

                if (item is TreeListHeaderItem)
                {
                    if (cellNum == 0)
                    {
                        td.ColumnSpan = maxLevel + 1;
                    }
                }
                else if (dataItem != null)
                {
                    if (cellNum == dataItem.HierarchyIndex.NestedLevel)
                    {
                        cellText = GetButtonCellText(dataItem);
                    }
                    else if (cellNum == dataItem.HierarchyIndex.NestedLevel + 1)
                    {
                        td.ColumnSpan = maxLevel - dataItem.HierarchyIndex.NestedLevel;
                    }
                }

                SetDocxCellText(td, GetCellText(cellText));
            }
        }
        GenerateFile(flowDoc, fileName);
    }

    private void SetDocxCellText(TableCell td, string cellText)
    {
        td.Blocks.AddParagraph().Inlines.AddRun(GetCellText(cellText));
    }

    private void ExportToXlsx()
    {
        string fileName = "RadTreeList.xlsx";
        string filePath = Server.MapPath(fileName);

        PrepareForExport();

        Workbook workbook = new Workbook();
        Worksheet worksheet = workbook.Worksheets.Add();
        TreeListItem item;

        var treeListItems = RadTreeList1.GetItems(supportedItemTypes);
        int maxLevel = RadTreeList1.GetMaximumNestedLevel();

        for (int itemNum = 0; itemNum < treeListItems.Length; itemNum++)
        {
            item = treeListItems[itemNum];

            for (int cellNum = 0; cellNum < item.Cells.Count; cellNum++)
            {
                var currentCell = item.Cells[cellNum];
                var dataItem = item as TreeListDataItem;

                CellSelection excelCell = null;
                if (item is TreeListHeaderItem)
                {
                    if (cellNum == 0)
                    {
                        excelCell = worksheet.Cells[itemNum, 0, itemNum, maxLevel + 1];
                        excelCell.Merge();
                    }
                    else
                    {
                        excelCell = worksheet.Cells[itemNum, cellNum + maxLevel + 1];
                    }
                }
                else if (dataItem != null)
                {
                    if (cellNum == dataItem.HierarchyIndex.NestedLevel + 1)
                    {
                        excelCell = worksheet.Cells[itemNum, cellNum, itemNum, maxLevel + 1];
                        excelCell.Merge();
                    }
                    else if (cellNum == dataItem.HierarchyIndex.NestedLevel)
                    {
                        worksheet.Cells[itemNum, cellNum].SetValue(GetButtonCellText(dataItem));
                    }
                    else if (cellNum > 0)
                    {
                        excelCell = worksheet.Cells[itemNum, cellNum + maxLevel - dataItem.HierarchyIndex.NestedLevel];
                    }
                }
                if (excelCell != null)
                {
                    excelCell.SetValue(GetCellText(currentCell.Text));
                }
            }
        }

        GenerateFile(workbook, fileName);
    }

    private string GetButtonCellText(TreeListDataItem dataItem)
    {
        string buttonText = String.Empty;

        if (dataItem.CanExpand)
        {
            buttonText = dataItem.Expanded ? "-" : "+";
        }

        return buttonText;
    }

    private void GenerateFile(object structure, string fileName)
    {
        RadFlowDocument flowDoc = structure as RadFlowDocument;
        Workbook workbook = structure as Workbook;

        byte[] output;

        using (MemoryStream ms = new MemoryStream())
        {
            if (flowDoc != null)
            {
                DocxFormatProvider provider = new DocxFormatProvider();
                provider.Export(flowDoc, ms);
            }
            else
            {
                XlsxFormatProvider provider = new XlsxFormatProvider();
                provider.Export(workbook, ms);
            }
            ms.Position = 0;
            output = ms.ToArray();
        }

        Response.ContentType = flowDoc != null ? "application/msword" : "application/excel";
        Response.AddHeader("content-disposition", "attachment; filename=" + fileName);
        Response.BinaryWrite(output);
        Response.Flush();
        Response.Close();
    }

    private string GetCellText(string text)
    {
        return text.Replace("&nbsp;", "");
    }

    private void PrepareForExport()
    {
        if (CheckBox1.Checked)
        {
            RadTreeList1.ExpandAllItems();
        }

        RadTreeList1.AllowPaging = false;
        RadTreeList1.Rebind();
    }

    protected void RadTreeList1_NeedDataSource(object sender, TreeListNeedDataSourceEventArgs e)
    {
        List<dynamic> dataList = new List<dynamic>();
        dataList.Add(new { Index = "1", ParentIndex = "", Name = "John" });
        dataList.Add(new { Index = "2", ParentIndex = "", Name = "Peter" });
        dataList.Add(new { Index = "3", ParentIndex = "", Name = "Michael" });
        dataList.Add(new { Index = "4", ParentIndex = "", Name = "Todd" });
        dataList.Add(new { Index = "5", ParentIndex = "", Name = "Brandon" });
        dataList.Add(new { Index = "6", ParentIndex = "", Name = "George" });
        dataList.Add(new { Index = "7", ParentIndex = "2", Name = "Jonathan" });
        dataList.Add(new { Index = "8", ParentIndex = "5", Name = "Daniel" });
        dataList.Add(new { Index = "9", ParentIndex = "2", Name = "Shawn" });
        dataList.Add(new { Index = "10", ParentIndex = "2", Name = "Bob" });
        dataList.Add(new { Index = "11", ParentIndex = "4", Name = "Casey" });
        dataList.Add(new { Index = "12", ParentIndex = "8", Name = "Adam" });
        dataList.Add(new { Index = "13", ParentIndex = "3", Name = "Carl" });
        dataList.Add(new { Index = "14", ParentIndex = "4", Name = "Corey" });
        dataList.Add(new { Index = "15", ParentIndex = "5", Name = "Kevin" });
        dataList.Add(new { Index = "16", ParentIndex = "9", Name = "Richard" });

        RadTreeList1.DataSource = dataList;
    }

    protected void Button2_Click(object sender, EventArgs e)
    {
        ExportToXlsx();
    }
    protected void Button1_Click(object sender, EventArgs e)
    {
        ExportToDocx();
    }
}